/***********************************************************************
 @C{
    filename        = "kept_lse.c",
    version         = "1.0",
    date            = "17 July 1992",
    time            = "13:11:40.16 BST",
    author          = "Norman Gray",
    address         = "Department of Physics
                       Open University
                       Milton Keynes
                       MK7 6AA, UK",
    e-mail          = "N.O.Gray@open.ac.uk (Internet)",
    telephone       = "+44 908 652279",
    FAX             = "+44 908 653744",
    archived        = "Aston, SHSU",
    keywords        = "LSE, TPU",
    abstract        = "This utility allows LSE to be kept running in a
                       subprocess, and allows the user to jump back and
                       forth between it and the main process.  To avoid
                       hogging resources, the unused subprocess is
                       suspended.",
    codetable       = "ISO/Ascii",
    checksum        = "27582 177 640 4944",
    docstring       = "Keep LSE running in a subprocess
                       
                       The checksum field above contains a CRC-16
                       checksum as the first value, followed by the
                       equivalent of the standard UNIX wc (word 
                       count) utility output of lines, words, and
                       characters.  This is produced by Robert 
                       Solovay's checksum utility."
}
***********************************************************************/
/*	 
**  This routine, tpu$calluser, is of the format which permits it to be called
**  by the TPU routine CALL_USER.
**
**  This routine may be called with  command_num  equal to
**	0 : suspend the parent process;
**	1 : resume the parent process;
**	2 : translate the job logical name given in the argument, and returns
**	    the translation.  If no translation is found, it returns a null
**	    string.
**	3 : define the job logical LSE$CURRENT_FILE to be the string argument
**
**  It returns any system-service errors, or SS$_NORMAL on success.
**
**  It seems that there is no condition handler present in TPU, as any attempt
**  at using lib$signal causes mayhem.  We must therefore establish our own
**  condition handler - use lib$sig_to_ret, which handles a condition and
**  returns it as the return value of the function.
*/	 

#include descrip
#include lnmdef
#include jpidef
#include ssdef
#include string




/* this one works, but is less good than lib$sig_to_ret */
#if 0
int handler (int sigargs[], int mechargs[])
{
printf ("%c%c%cHANDLER\n", 7, 7, 7);

sys$unwind (mechargs[2]);
}
#endif





int lib$sig_to_ret();



int tpu$calluser (int *command_num, struct dsc$descriptor_s *string,
    struct dsc$descriptor_s *return_string)
{
static unsigned long parent_pid = 0;

static $DESCRIPTOR (Jobtable, "LNM$JOB");
unsigned long status;


lib$establish (lib$sig_to_ret);
/* tpu$calluser = SS$_NORMAL; seems to be the default */


if (!parent_pid) {	/* first time through, get parent pid */
    int jpitem = JPI$_OWNER;

    if ((status = lib$getjpi (&jpitem, 0, 0, &parent_pid)) != SS$_NORMAL)
	return status;
    }




/* which command was passed to the routine? */
switch (*command_num){
case 0:	    /* suspend the parent */
    status = sys$suspnd (&parent_pid, 0, 0);
    break;

case 1:	    /* resume the parent */
    status = sys$resume (&parent_pid, 0);
    break;

case 2:
    /* translate the logical name, returning a null string if the	    */
    /* return status is SS$_NOLOGNAM					    */
    {
    static unsigned short retlength;

    static char buf[LNM$C_NAMLENGTH];	    /* return of logical name */
    register int i;			    /* (string) array index */

    static struct {			    /* item list for SYS$TRNLNM */
	unsigned short	buflen;
	unsigned short	item_code;
	char		*buffer;
	unsigned short  *return_count;
	} itemlist[2] = {
	    {LNM$C_NAMLENGTH, LNM$_STRING, buf, &retlength},
	    {0, 0, 0, 0}
	    };


    if ((status = sys$trnlnm (0, &Jobtable, string, 0, itemlist))
	    != SS$_NORMAL)
	if (status == SS$_NOLOGNAM)
	    retlength = 0;
	else {
	    break; }

    /* now allocate some space for the return value, then copy |buf| into   */
    /* the return descriptor						    */

    if ((status = lib$sget1_dd (&retlength, return_string)) != SS$_NORMAL) {
	break; }

    if (retlength > 0)
	/* Copy buf --> return string.					    */
	strncpy (return_string->dsc$a_pointer, buf, retlength);

    break;
    }

case 3:
    /* Define the Job logical LSE$CURRENT_FILE to have the value |string|   */
    {
    static $DESCRIPTOR (Logname, "LSE$CURRENT_FILE");

    status = lib$set_logical (&Logname, string, &Jobtable);

    break;
    }

case 4:
    /* for debugging condition handling */
    lib$signal (SS$_ACCVIO);
    break;

default:    /* unknown command */
    status = 0;
}	/* end of switch on (*command_num)*/



return status;

}
