$ !!! ======================================================================
$ !!!  @VAX-VMS-shell{
$ !!!     filename        = "kept_lse.com",
$ !!!     version         = "1.0",
$ !!!     date            = "17 July 1992",
$ !!!     time            = "13:12:47.25 BST",
$ !!!     author          = "Norman Gray",
$ !!!     address         = "Department of Physics
$ !!!                        Open University
$ !!!                        Milton Keynes
$ !!!                        MK7 6AA, UK",
$ !!!     e-mail          = "N.O.Gray@open.ac.uk (Internet)",
$ !!!     telephone       = "+44 908 652279",
$ !!!     FAX             = "+44 908 653744",
$ !!!     archived        = "Aston, SHSU",
$ !!!     keywords        = "LSE, TPU",
$ !!!     abstract        = "This utility allows LSE to be kept running in a
$ !!!                        subprocess, and allows the user to jump back and
$ !!!                        forth between it and the main process.  To avoid
$ !!!                        hogging resources, the unused subprocess is
$ !!!                        suspended.",
$ !!!     codetable       = "ISO/Ascii",
$ !!!     checksum        = "10126 160 783 5414",
$ !!!     docstring       = "Keep LSE running in a subprocess
$ !!!                        
$ !!!                        The checksum field above contains a CRC-16
$ !!!                        checksum as the first value, followed by the
$ !!!                        equivalent of the standard UNIX wc (word 
$ !!!                        count) utility output of lines, words, and
$ !!!                        characters.  This is produced by Robert 
$ !!!                        Solovay's checksum utility."
$ !!! }
$ !!! ======================================================================
$!  +
$!  This routine will spawn a subprocess and run LSE in it.  When control
$!  returns to this routine, it checks that the subprocess is still there (that
$!  is, that the user ATTACHed from the subrpocess back to the main one),
$!  suspends the subprocess and exits.  When it is invoked again, it sees that
$!  the subprocess already exists, unsuspends it, and reattaches to it.
$!  \-
$
$   on error then goto finishoff
$   on control_y then goto finishoff
$
$   klse_root := phut:[util.com.kept_lse]
$!   klse_root := me:[coms.commands.klse]
$
$   delete = "delete"
$
$!  All names defined during the procedure should be deassigned at the end,
$!  after `finishoff'.  They shouldn't be deassigned in the body, in case ^Y
$!  comes at an unfortunate time
$   deassignments = 0
$   deas_command = 1	    ! set if KLSE_COMMAND has been defined
$   deas_lse_command_file = 2
$   deas_calluser = 4
$
$   if f$extract (0, 1, p1) .eqs. "-"
$   then
$	comchar = f$extract (1, 1, p1)
$	if (comchar .eqs. "D")
$	then
$	    define/job klse_command "D''f$environment("default")'"
$	else
$	    define/job klse_command "''f$edit(f$extract (1, 99, p1),"upcase")'"
$	endif
$	deassignments = deassignments .or. deas_command
$	fn = ""
$   else
$	fn = p1
$   endif
$
$   if fn .eqs. "?"	! show whether there is a kept lse running
$   then
$	if f$type (kept_lse_name) .eqs. ""
$	then
$	    write sys$output "No kept LSE running"
$	    EXIT	! JUMP OUT
$	else
$	    define/job klse_command "B"	    ! enter LSE and display buffers
$	    deassignments = deassignments .or. deas_command
$	    fn = ""
$	endif
$   endif
$   
$   if f$type (kept_lse_name) .eqs. ""	    ! no subprocess present
$   then
$
$	kept_lse_name == "KLSE_" + f$getjpi ("", "pid")	! prc name max 15 chars
$
$	if fn .eqs. "" then read sys$command/prompt="Filename: " fn
$
$	lse_command_file = f$trnlnm ("lse$command")
$	if lse_command_file .nes. ""
$	then
$	    define/job klse_tpu_command 'lse_command_file
$	    deassignments = deassignments .or. deas_lse_command_file
$	endif
$
$	define/process tpu$calluser -
	    'klse_root'kept_lse
$	deassignments = deassignments .or. deas_calluser
$	saved_controls = f$environment ("control")
$	set nocontrol=y
$
$	spawn/wait/process='kept_lse_name'/logical/nosymbol/nolog -
	    lsedit 'fn -
	    /command='klse_root'kept_lse.tpu
$
$	set control=('saved_controls')
$	
$   else
$
$	if fn .nes. ""
$	then
$	    define/job klse_command "R''fn'"	! read in this file
$	    deassignments = deassignments .or. deas_command
$	endif
$
$	set process 'kept_lse_name'/resume
$	attach 'kept_lse_name
$
$   endif
$
$
$finishoff:
$   if (deassignments.and.deas_lse_command_file) .ne. 0 then -
	$ deassign/job klse_tpu_command
$   if (deassignments.and.deas_command) .ne. 0 then -
	$ deassign/job klse_command
$   if (deassignments.and.deas_calluser) .ne. 0 then -
	$ deassign/process tpu$calluser
$
$   on error then exit ! restore default handler
$   on control_y then continue	    ! confusion will result if  kept_lse_name is
$				    ! wrong, or present when it shouldn't be
$
$   ! now check to see if it's still there
$   context = ""
$   temp = f$context ("PROCESS", context, "PRCNAM", kept_lse_name, "EQL")
$   pid = f$pid (context)
$
$   if pid .eqs. ""
$   then 	    ! nothing doing
$	delete/symbol/global kept_lse_name
$   else
$	! it is still there
$	set process 'kept_lse_name'/suspend  ! suspend the process
$   endif
$
$   ! supersede any process-table lse$current_file
$   cfile = f$trnlnm ("lse$current_file", "lnm$job")
$   if cfile .nes. ""
$   then
$	define/nolog/process lse$current_file 'cfile
$	deassign/job lse$current_file
$   endif
$
$
$   exit
